//////////////////////////////////////////////////////////////////////////
//
//  UltraDefrag - a powerful defragmentation tool for Windows NT.
//  Copyright (c) 2007-2013 Dmitri Arkhangelski (dmitriar@gmail.com).
//  Copyright (c) 2010-2013 Stefan Pendl (stefanpe@users.sourceforge.net).
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////

/**
 * @file help.cpp
 * @brief Help screen.
 * @addtogroup Help
 * @{
 */

// Ideas by Stefan Pendl <stefanpe@users.sourceforge.net>
// and Dmitri Arkhangelski <dmitriar@gmail.com>.

// =======================================================================
//                            Declarations
// =======================================================================

#include "main.h"

// =======================================================================
//                            Help screen
// =======================================================================

void show_help(void)
{
    printf(
        "===============================================================================\n"
        VERSIONINTITLE " - a powerful disk defragmentation tool for Windows NT\n"
        "Copyright (c) UltraDefrag Development Team, 2007-2013.\n"
        "\n"
        "===============================================================================\n"
        "This program is free software; you can redistribute it and/or\n"
        "modify it under the terms of the GNU General Public License\n"
        "as published by the Free Software Foundation; either version 2\n"
        "of the License, or (at your option) any later version.\n"
        "\n"
        "This program is distributed in the hope that it will be useful,\n"
        "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
        "GNU General Public License for more details.\n"
        "\n"
        "You should have received a copy of the GNU General Public License\n"
        "along with this program; if not, write to the Free Software\n"
        "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n"
        "===============================================================================\n"
        "\n"
        "Usage: udefrag [command] [options] [driveletter:] [path(s)]\n"
        "\n"
        "  The default action is to display this help message.\n"
        "\n"
        "Commands:\n"
        "  -a,  --analyze                      Analyze disk\n"
        "       --defragment                   Defragment disk\n"
        "  -o,  --optimize                     Optimize disk space\n"
        "  -q,  --quick-optimize               Perform quick optimization\n"
        "       --optimize-mft                 Place MFT fragments as close\n"
        "                                      to each other as possible\n"
        "  -l,  --list-available-volumes       List disks available\n"
        "                                      for defragmentation,\n"
        "                                      except removable media\n"
        "  -la, --list-available-volumes=all   List all available disks\n"
        "  -h,  --help                         Show this help screen\n"
        "  -?                                  Show this help screen\n"
        "\n"
        "  The commands are exclusive and can't be combined with each other.\n"
        "  If no command is specified it will defragment the disk.\n"
        "\n"
        "Options:\n"
        "  -r,  --repeat                       Repeat action until nothing left to move.\n"
        "  -b,  --use-system-color-scheme      Use system (usually black/white)\n"
        "                                      color scheme instead of the green color.\n"
        "  -p,  --suppress-progress-indicator  Hide progress indicator and cluster map.\n"
        "  -v,  --show-volume-information      Show disk information after a job.\n"
        "  -m,  --show-cluster-map             Show map representing clusters\n"
        "                                      on the disk.\n"
        "       --map-border-color=color       Set cluster map border color.\n"
        "                                      Available color values: black, white,\n"
        "                                      red, green, blue, yellow, magenta, cyan,\n"
        "                                      darkred, darkgreen, darkblue, darkyellow,\n"
        "                                      darkmagenta, darkcyan, gray.\n"
        "                                      Yellow color is used by default.\n"
        "       --map-symbol=x                 Set a character used for the map drawing.\n"
        "                                      There are two accepted formats:\n"
        "                                      a character may be typed directly,\n"
        "                                      or its hexadecimal number may be used.\n"
        "                                      For example, --map-symbol=0x1 forces\n"
        "                                      UltraDefrag to use a smile character\n"
        "                                      for the map drawing.\n"
        "                                      Valid numbers are in range: 0x1 - 0xFF\n"
        "                                      \'%%\' symbol is used by default.\n"
        "       --map-rows=n                   Number of rows in cluster map.\n"
        "                                      Default value is 10.\n"
        "       --map-symbols-per-line=n       Number of map symbols\n"
        "                                      containing in each row of the map.\n"
        "                                      Default value is 68.\n"
        "       --use-entire-window            Expand cluster map to use entire\n"
        "                                      console window.\n"
        "       --wait                         Wait until already running instance\n"
        "                                      of udefrag.exe tool completes before\n"
        "                                      starting the job (useful for\n"
        "                                      the scheduled defragmentation).\n"
        "       --shellex                      This option forces to list objects\n"
        "                                      to be processed and display a prompt\n"
        "                                      to hit any key after a job completion.\n"
        "                                      It is intended for use in Explorer\'s\n"
        "                                      context menu handler.\n"
        "\n"
        "Drive letter:\n"
        "  It is possible to specify multiple drive letters, like this:\n\n"
        "  udefrag c: d: x:\n\n"
        "  Also the following keys can be used instead of the drive letter:\n\n"
        "  --all                               Process all available disks.\n"
        "  --all-fixed                         Process all disks except removable.\n"
        "\n"
        "Path:\n"
        "  It is possible to specify multiple paths, like this:\n\n"
        "  udefrag \"C:\\Documents and Settings\" C:\\WINDOWS\\WindowsUpdate.log\n\n"
        "  Paths including spaces must be enclosed in double quotes (\").\n"
        "  Relative and absolute paths are supported.\n"
        "\n"
        "Accepted environment variables:\n"
        "\n"
        "  UD_IN_FILTER                        List of files to be included\n"
        "                                      in defragmentation process. Patterns\n"
        "                                      must be separated by semicolons.\n"
        "\n"
        "  UD_EX_FILTER                        List of files to be excluded from\n"
        "                                      defragmentation process. Patterns\n"
        "                                      must be separated by semicolons.\n"
        "\n"
        "  UD_FRAGMENT_SIZE_THRESHOLD          Eliminate only fragments smaller\n"
        "                                      than specified.\n"
        "                                      The following size suffixes are accepted:\n"
        "                                      KB, MB, GB, TB, PB, EB.\n"
        "\n"
        "  UD_FILE_SIZE_THRESHOLD              Exclude all files larger than specified.\n"
        "                                      The following size suffixes are accepted:\n"
        "                                      KB, MB, GB, TB, PB, EB.\n"
        "\n"
        "  UD_OPTIMIZER_FILE_SIZE_THRESHOLD    In optimization sort out files\n"
        "                                      smaller than specified only.\n"
        "                                      The following size suffixes are accepted:\n"
        "                                      KB, MB, GB, TB, PB, EB.\n"
        "                                      The default value is 20MB.\n"
        "\n"
        "  UD_FRAGMENTS_THRESHOLD              Exclude all files which have less number\n"
        "                                      of fragments than specified.\n"
        "\n"
        "  UD_SORTING                          Set the file sorting method for the disk\n"
        "                                      optimization. PATH is used by default,\n"
        "                                      it forces to sort files by their paths.\n"
        "                                      Four more options are available:\n"
        "                                      SIZE (sort by size), C_TIME (sort by\n"
        "                                      creation time), M_TIME (sort by last\n"
        "                                      modification time) and A_TIME (sort by\n"
        "                                      last access time).\n"
        "\n"
        "  UD_SORTING_ORDER                    Set the file sorting order for the disk\n"
        "                                      optimization. ASC (ascending) is used\n"
        "                                      by default. DESC (descending) forces\n"
        "                                      to sort files in descending order.\n"
        "\n"
        "  UD_FRAGMENTATION_THRESHOLD          Set the fragmentation threshold,\n"
        "                                      in percents. When the disk fragmentation\n"
        "                                      level is below the specified value, all\n"
        "                                      the disk processing jobs except of the\n"
        "                                      MFT optimization will be not performed.\n"
        "\n"
        "  UD_TIME_LIMIT                       When the specified time interval elapses\n"
        "                                      the job will be stopped automatically.\n"
        "                                      The following time format is accepted:\n"
        "                                      Ay Bd Ch Dm Es. Here A,B,C,D,E represent\n"
        "                                      any integer numbers, y,d,h,m,s - suffixes\n"
        "                                      used for years, days, hours, minutes\n"
        "                                      and seconds.\n"
        "\n"
        "  UD_REFRESH_INTERVAL                 Progress refresh interval,\n"
        "                                      in milliseconds.\n"
        "                                      The default value is 100.\n"
        "\n"
        "  UD_DISABLE_REPORTS                  If this environment variable is set\n"
        "                                      to 1 (one), no file fragmentation\n"
        "                                      reports will be generated.\n"
        "\n"
        "  UD_DBGPRINT_LEVEL                   Control amount of the debugging output.\n"
        "                                      NORMAL is used by default, DETAILED\n"
        "                                      may be used to collect information for\n"
        "                                      the bug report, PARANOID turns on\n"
        "                                      a really huge amount of debugging\n"
        "                                      information.\n"
        "\n"
        "  UD_LOG_FILE_PATH                    If this variable is set, it should\n"
        "                                      contain the path (including filename)\n"
        "                                      to the log file to save debugging output\n"
        "                                      into. If the variable is not set, no\n"
        "                                      logging to the file will be performed.\n"
        "\n"
        "  UD_DRY_RUN                          If this environment variable is set\n"
        "                                      to 1 (one) the files are not physically\n"
        "                                      moved.\n"
        "                                      This allows testing the algorithm without\n"
        "                                      changing the content of the disk.\n"
        "\n"
        "Note:\n"
        "  All the environment variables are ignored when the --shellex switch is\n"
        "  on the command line. Instead of taking environment variables into account\n"
        "  the program interpretes the %%UD_INSTALL_DIR%%\\options\\guiopts.lua file.\n"
        "\n"
        "Examples:\n"
        "  set UD_IN_FILTER=*windows*;*winnt*  include only paths, which include either\n"
        "                                      'windows' or 'winnt' words.\n"
        "\n"
        "  set UD_FILE_SIZE_THRESHOLD=50MB     process only files below 50 MB in size.\n"
        "\n"
        "  set UD_FRAGMENTS_THRESHOLD=6        process only files with more than\n"
        "                                      '6' fragments.\n"
        "\n"
        "  set UD_TIME_LIMIT=6h 30m            terminate processing after 6 hours\n"
        "                                      and 30 minutes.\n"
        "\n"
        "  set UD_LOG_FILE_PATH=C:\\Windows\\Temp\\defrag_native.log\n"
        "                                      save debugging information to the\n"
        "                                      specified file.\n"
        "\n"
        "  udefrag C:                          defragment drive C:\n"
        "\n"
        "  udefrag -la                         list all available drives.\n"
        "\n"
        "  udefrag -a --all-fixed              analyze all drives excluding\n"
        "                                      removable ones.\n"
        "\n"
        "  udefrag -o E: K:                    optimize drives E: and K:\n"
        "\n"
        "  udefrag --optimize-mft O: Q:        optimize the MFT of drives O: and Q:\n"
        "\n"
        "  udefrag C:\\Windows\\WindowsUpdate.log \"C:\\Program Files\"\n"
        "                                      defragment the specified file and folder.\n"
        "\n"
        );
}

/** @} */
